from Param cimport *

cdef extern from "<OpenMS/FORMAT/ParamXMLFile.h>" namespace "OpenMS":

    cdef cppclass ParamXMLFile:

        ParamXMLFile() nogil except +
        void load(String, Param &) nogil except+
        void store(String, Param &) nogil except+



from Types cimport *
from String cimport *
from Peak1D cimport *
from InstrumentSettings cimport *
from SourceFile cimport *
from PeptideIdentification cimport *
from Precursor cimport *
from DataProcessing cimport *
from Product cimport *
from AcquisitionInfo cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/SpectrumSettings.h>" namespace "OpenMS":

    cdef cppclass SpectrumSettings(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        SpectrumSettings()    nogil except +
        void unify(SpectrumSettings)    nogil except +
        int  getType()    nogil except +
        void setType(SpectrumType)    nogil except +
        String getNativeID()    nogil except +
        void setNativeID(String)    nogil except +
        String getComment()    nogil except +
        void setComment(String)    nogil except +

        InstrumentSettings getInstrumentSettings()    nogil except +
        void setInstrumentSettings(InstrumentSettings)    nogil except +

        AcquisitionInfo getAcquisitionInfo()    nogil except +
        void setAcquisitionInfo(AcquisitionInfo)    nogil except +

        SourceFile getSourceFile()    nogil except +
        void setSourceFile(SourceFile)    nogil except +

        libcpp_vector[Precursor] getPrecursors() nogil except +
        void setPrecursors(libcpp_vector[Precursor])   nogil except +

        libcpp_vector[Product] getProducts() nogil except +
        void setProducts(libcpp_vector[Product])   nogil except +

        libcpp_vector[PeptideIdentification] getPeptideIdentifications() nogil except +
        void setPeptideIdentifications(libcpp_vector[PeptideIdentification])   nogil except +

        libcpp_vector[ shared_ptr[DataProcessing] ] getDataProcessing() nogil except +
        void setDataProcessing(libcpp_vector[ shared_ptr[DataProcessing] ])   nogil except +

cdef extern from "<OpenMS/METADATA/SpectrumSettings.h>" namespace "OpenMS::SpectrumSettings":

    cdef enum SpectrumType:
        # wrap-attach:
        #     SpectrumSettings
        UNKNOWN,  PEAKS, RAWDATA, SIZE_OF_SPECTRUMTYPE




import pygraphviz as pgv
import sys

def main(argv):
    if len(argv) not in (2,3,4,5):
        print 'Usage: draw_dot.py <graph .dot> <output image file> [layout, default is neato; optionally multiple layouts separated by ","] [overlap mode {scale, false}, default is false] [label edges {0,1}]'
    else:
        layouts = ['neato']
        if len(argv) >= 3:
            layouts = argv[2].split(',')

        G = pgv.AGraph(argv[0])

        overlap='false'
        if len(argv) >= 4:
            overlap=argv[3]

        label_edges=True
        if len(argv) >= 5:
            label_edges = bool(int(argv[4]))
        if not label_edges:
            for e in G.edges():
                e.attr['label'] = ' '
        
        for lo in layouts[:-1]:
            G.layout(prog=lo)
        G.layout(prog=layouts[-1], args='-Goverlap=' + overlap + ' -Gsplines=true')

        G.draw(argv[1])

if __name__ == "__main__":
    main(sys.argv[1:])



from Types cimport *
from libcpp.map cimport map as libcpp_map
from libcpp.vector cimport vector as libcpp_vector
from ProteinIdentification cimport *
from PeptideIdentification cimport *
from XMLHandler cimport *
from XMLFile cimport *
from AASequence cimport *

cdef extern from "<OpenMS/FORMAT/PepXMLFileMascot.h>" namespace "OpenMS":
    
    cdef cppclass PepXMLFileMascot :

        PepXMLFileMascot() nogil except +
        PepXMLFileMascot(PepXMLFileMascot) nogil except + #wrap-ignore

        # TODO map
        # void load(const String & filename, libcpp_map[ String, libcpp_vector[ AASequence ] ] & peptides) nogil except +




from Types cimport *
from String cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/MRMFeatureQC.h>" namespace "OpenMS":

    cdef cppclass MRMFQC_ComponentGroupQCs "OpenMS::MRMFeatureQC::ComponentGroupQCs":
        MRMFQC_ComponentGroupQCs()
        MRMFQC_ComponentGroupQCs(MRMFQC_ComponentGroupQCs &) # no-wrap

        String component_group_name
        double retention_time_l
        double retention_time_u
        double intensity_l
        double intensity_u
        double overall_quality_l
        double overall_quality_u
        Int n_heavy_l
        Int n_heavy_u
        Int n_light_l
        Int n_light_u
        Int n_detecting_l
        Int n_detecting_u
        Int n_quantifying_l
        Int n_quantifying_u
        Int n_identifying_l
        Int n_identifying_u
        Int n_transitions_l
        Int n_transitions_u
        String ion_ratio_pair_name_1
        String ion_ratio_pair_name_2
        double ion_ratio_l
        double ion_ratio_u
        String ion_ratio_feature_name
        # libcpp_map[String,libcpp_pair[double,double]] meta_value_qc
        ## currently not supported

    cdef cppclass MRMFQC_ComponentQCs "OpenMS::MRMFeatureQC::ComponentQCs":
        MRMFQC_ComponentQCs()
        MRMFQC_ComponentQCs(MRMFQC_ComponentQCs &) # no-wrap

        String component_name 
        double retention_time_l
        double retention_time_u
        double intensity_l
        double intensity_u
        double overall_quality_l
        double overall_quality_u
        # libcpp_map[String,libcpp_pair[double,double]] meta_value_qc
        ## currently not supported

    cdef cppclass MRMFQC_ComponentGroupPairQCs "OpenMS::MRMFeatureQC::ComponentGroupPairQCs":
        MRMFQC_ComponentGroupPairQCs()
        MRMFQC_ComponentGroupPairQCs(MRMFQC_ComponentGroupPairQCs &) # no-wrap

        String component_group_name 
        String resolution_pair_name 
        double resolution_l
        double resolution_u
        double rt_diff_l
        double rt_diff_u

    cdef cppclass MRMFeatureQC:

        MRMFeatureQC() nogil except +
        MRMFeatureQC(MRMFeatureQC &) nogil except +
        
        libcpp_vector[MRMFQC_ComponentQCs] component_qcs
        libcpp_vector[MRMFQC_ComponentGroupQCs] component_group_qcs
        libcpp_vector[MRMFQC_ComponentGroupPairQCs] component_group_pair_qcs




from libcpp.string cimport string as libcpp_string
from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from libcpp cimport bool
from DataValue cimport *
from String cimport *
from StringList cimport *
from ParamNode cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/Param.h>" namespace "OpenMS::Param":

    cdef cppclass ParamEntry:

        String name
        String description
        DataValue value
        libcpp_set[String] tags
        libcpp_vector[String] valid_strings
        double  max_float
        double  min_float
        Int  max_int
        Int  min_int

        ParamEntry() nogil except +
        ParamEntry(ParamEntry) nogil except +
        ParamEntry(String n, DataValue v, String d, StringList t) nogil except +
        ParamEntry(String n, DataValue v, String d) nogil except +

        bool isValid(String &message) nogil except +
        bool operator==(ParamEntry) nogil except +




#!/usr/bin/env python
# -*- coding: utf-8  -*-
from __future__ import print_function

import copy
import os

from pyopenms import String as s
from pyopenms import *

try:
    long
except NameError:
    long = int

from functools import wraps


def report(f):
    @wraps(f)
    def wrapper(*a, **kw):
        print("run b", f.__name__)
        f(*a, **kw)
    return wrapper

@report
def testElementTutorial():

    edb = ElementDB()

    assert edb.hasElement("O")
    assert edb.hasElement("S")

    oxygen = edb.getElement("O")
    assert oxygen.getName() == "Oxygen"
    assert oxygen.getSymbol() == "O"
    oxygen.getMonoWeight()
    isotopes = oxygen.getIsotopeDistribution()

    sulfur = edb.getElement("S")
    sulfur.getName()
    sulfur.getSymbol()
    sulfur.getMonoWeight()
    isotopes = sulfur.getIsotopeDistribution()
    for iso in isotopes.getContainer():
        print (iso.getMZ(), ":", iso.getIntensity())

@report
def testEmpiricalFormulaTutorial():

    print("testEmpiricalFormulaTutorial")
    methanol = EmpiricalFormula("CH3OH")
    water = EmpiricalFormula("H2O")
    ethanol = EmpiricalFormula("CH2" + methanol.toString())
    wm = water + methanol
    print(wm)
    print(wm.getElementalComposition())

    # TODO: dicts are still using bytes!
    wm = water + methanol # only in pyOpenMS 2.4
    m = wm.getElementalComposition()
    assert m[b"C"] == 1
    assert m[b"H"] == 6
    assert m[b"O"] == 2

    wm = EmpiricalFormula("CH3OH") + EmpiricalFormula("H2O")

    isotopes = wm.getIsotopeDistribution( CoarseIsotopePatternGenerator(3) )
    for iso in isotopes.getContainer():
        print (iso.getMZ(), ":", iso.getIntensity())

    isotopes = wm.getIsotopeDistribution( CoarseIsotopePatternGenerator(3, True) )
    for iso in isotopes.getContainer():
        print (iso.getMZ(), ":", iso.getIntensity())

@report
def testResidueTutorial():

    lys = ResidueDB().getResidue("Lysine")
    assert lys.getName() == "Lysine"
    lys.getThreeLetterCode() == "LYS"
    lys.getOneLetterCode() == "K"
    lys.getAverageWeight(Residue.ResidueType.Full)
    assert abs(lys.getMonoWeight(Residue.ResidueType.Full) - 146.1055284466) < 1e-5
    lys.getPka()

@report
def testAASequenceTutorial():

    seq = AASequence.fromString("DFPIANGER")
    prefix = seq.getPrefix(4)
    suffix = seq.getSuffix(5)
    concat = seq + seq

    print(seq)
    print(concat)
    print(suffix)
    seq.getMonoWeight() # weight of M
    seq.getMonoWeight(Residue.ResidueType.Full, 2) # weight of M+2H
    mz = seq.getMonoWeight(Residue.ResidueType.Full, 2) / 2.0 # m/z of M+2H
    concat.getMonoWeight()

    print("Monoisotopic m/z of (M+2H)2+ is", mz)

    seq_formula = seq.getFormula()
    print("Peptide", seq, "has molecular formula", seq_formula)
    print("="*35)

    isotopes = seq_formula.getIsotopeDistribution( CoarseIsotopePatternGenerator(6) )
    for iso in isotopes.getContainer():
            print ("Isotope", iso.getMZ(), ":", iso.getIntensity())

    suffix = seq.getSuffix(3) # y3 ion "GER"
    print("="*35)
    print("y3 ion :", suffix)
    y3_formula = suffix.getFormula(Residue.ResidueType.YIon, 2) # y3++ ion
    suffix.getMonoWeight(Residue.ResidueType.YIon, 2) / 2.0 # CORRECT
    suffix.getMonoWeight(Residue.ResidueType.XIon, 2) / 2.0 # CORRECT
    suffix.getMonoWeight(Residue.ResidueType.BIon, 2) / 2.0 # INCORRECT
    assert str(y3_formula) == "C13H24N6O6", str(y3_formula)
    assert str(seq_formula) == "C44H67N13O15"


    print("y3 mz :", suffix.getMonoWeight(Residue.ResidueType.YIon, 2) / 2.0 )
    print(y3_formula)
    print(seq_formula)

    seq = AASequence.fromString("PEPTIDESEKUEM(Oxidation)CER")
    print(seq)
    print(seq.toString())
    print(seq.toUnmodifiedString())
    print(seq.toUniModString())
    print(seq.toBracketString())
    print(seq.toBracketString(False))

    print(AASequence.fromString("DFPIAM(UniMod:35)GER"))
    print(AASequence.fromString("DFPIAM[+16]GER"))
    print(AASequence.fromString("DFPIAM[+15.99]GER"))
    print(AASequence.fromString("DFPIAM[147]GER"))
    print(AASequence.fromString("DFPIAM[147.035405]GER"))

    s = AASequence.fromString(".(Dimethyl)DFPIAMGER.")
    print(s, s.hasNTerminalModification())
    s = AASequence.fromString(".DFPIAMGER.(Label:18O(2))")
    print(s, s.hasCTerminalModification())
    s = AASequence.fromString(".DFPIAMGER(Phospho).")
    print(s, s.hasCTerminalModification())


    bsa = FASTAEntry()
    bsa.sequence = "MKWVTFISLLLLFSSAYSRGVFRRDTHKSEIAHRFKDLGE"
    bsa.description = "BSA Bovine Albumin (partial sequence)"
    bsa.identifier = "BSA"
    alb = FASTAEntry()
    alb.sequence = "MKWVTFISLLFLFSSAYSRGVFRRDAHKSEVAHRFKDLGE"
    alb.description = "ALB Human Albumin (partial sequence)"
    alb.identifier = "ALB"

    entries = [bsa, alb]

    f = FASTAFile()
    f.store("example.fasta", entries)

    entries = []
    f = FASTAFile()
    f.load("example.fasta", entries)
    print( len(entries) )
    for e in entries:
      print (e.identifier, e.sequence)

@report
def testTheoreticalSpectrumGenTutorial():


    tsg = TheoreticalSpectrumGenerator()
    spec1 = MSSpectrum()
    spec2 = MSSpectrum()
    peptide = AASequence.fromString("DFPIANGER")
    # standard behavior is adding b- and y-ions of charge 1
    p = Param()
    p.setValue("add_b_ions", "false", "Add peaks of b-ions to the spectrum")
    tsg.setParameters(p)
    tsg.getSpectrum(spec1, peptide, 1, 1)
    p.setValue("add_b_ions", "true", "Add peaks of a-ions to the spectrum")
    p.setValue("add_metainfo", "true", "")
    tsg.setParameters(p)
    tsg.getSpectrum(spec2, peptide, 1, 2)
    print("Spectrum 1 has", spec1.size(), "peaks.")
    print("Spectrum 2 has", spec2.size(), "peaks.")

    # Iterate over annotated ions and their masses
    for ion, peak in zip(spec2.getStringDataArrays()[0], spec2):
        print(ion, peak.getMZ())



@report
def testDigestionTutorial():

    print("testDigestionTutorial")
    dig = ProteaseDigestion()
    dig.getEnzymeName() # Trypsin
    bsa = b'DEHVKLVNELTEFAKTCVADESHAGCEKSLHTLFGDELCKVASLRETYGDMADCCEKQEPERNECFLSHKDDSPDLPKLKPDPNTLCDEFKADEKKFWGKYLYEIARRHPYFYAPELLYYANKYNGVFQECCQAEDKGACLLPKIETMREKVLASSARQRLRCASIQKFGERALKAWSVARLSQKFPKAEFVEVTKLVTDLTKVHKECCHGDLLECADDRADLAKYICDNQDTISSKLKECCDKPLLEKSHCIAEVEKDAIPENLPPLTADFAEDKDVCKNYQEAKDAFLGSFLYEYSRRHPEYAVSVLLRLAKEYEATLEECCAKDDPHACYSTVFDKLKHLVDEPQNLIKQNCDQFEKLGEYGFQNALIVRYTRKVPQVSTPTLVEVSRSLGKVGTRCCTKPESERMPCTEDYLSLILNRLCVLHEKTPVSEKVTKCCTESLVNRRPCFSALTPDETYVPKAFDEKLFTFHADICTLPDTEKQIKKQTALVELLKHKPKATEEQLKTVMENFVAFVDKCCAADDKEACFAVEGPKLVVSTQTALA'
    bsa = AASequence.fromString(bsa)
    result = []
    dig.digest(bsa, result)
    print(result[4])
    len(result) # 74 peptides
    assert len(result) == 74

    names = []
    ProteaseDB().getAllNames(names)
    len(names) # 25 by default
    e = ProteaseDB().getEnzyme('Lys-C')
    e.getRegExDescription()
    e.getRegEx()

    dig = ProteaseDigestion()
    dig.setEnzyme('Lys-C')
    result = []
    dig.digest(bsa, result)
    print(result[4])
    len(result) # 53 peptides
    assert len(result) == 53

def gaussian(x, mu, sig):
    import numpy as np
    return np.exp(-np.power(x - mu, 2.) / (2 * np.power(sig, 2.)))

@report
def testDatastructuresTutorial():

    ###################################
    # Spectrum
    # ********
    ###################################

    spectrum = MSSpectrum()
    mz = range(1500, 500, -100)
    i = [0 for mass in mz]
    spectrum.set_peaks([mz, i])

    # Sort the peaks according to ascending mass-to-charge ratio
    spectrum.sortByPosition()

    # Iterate over spectrum of those peaks
    for p in spectrum:
        print(p.getMZ(), p.getIntensity())

    # More efficient peak access
    for mz, i in zip(*spectrum.get_peaks()):
        print(mz, i)

    # Access a peak by index
    print(spectrum[1].getMZ(), spectrum[1].getIntensity())


    ###################################
    ###################################

    spectrum = MSSpectrum()
    spectrum.setDriftTime(25) # 25 ms
    spectrum.setRT(205.2) # 205.2 s
    spectrum.setMSLevel(3) # MS3
    p = Precursor()
    p.setIsolationWindowLowerOffset(1.5)
    p.setIsolationWindowUpperOffset(1.5) 
    p.setMZ(600) # isolation at 600 +/- 1.5 Th
    p.setActivationEnergy(40) # 40 eV
    p.setCharge(4) # 4+ ion
    spectrum.setPrecursors( [p] )

    fda = FloatDataArray()
    fda.setName("Signal to Noise Array")
    fda.push_back(15)
    sda = StringDataArray()
    sda.setName("Peak annotation")
    sda.push_back("y15++")
    spectrum.setFloatDataArrays( [fda] )
    spectrum.setStringDataArrays( [sda] )
    spectrum.set_peaks( ([401.5], [900]) )

    # Store as mzML
    exp = MSExperiment()
    exp.addSpectrum(spectrum)
    spectrum = MSSpectrum()
    spectrum.set_peaks( ([1, 2], [1, 2]) )
    exp.addSpectrum(spectrum)
    MzMLFile().store("testfile.mzML", exp)

    ###################################
    # Peak Map
    # *********
    ###################################

    # The following examples creates a MSExperiment containing four MSSpectrum instances.
    exp = MSExperiment()
    for i in range(6):
        spectrum = MSSpectrum()
        spectrum.setRT(i)
        spectrum.setMSLevel(1)
        for mz in range(500, 900, 100):
          peak = Peak1D()
          peak.setMZ(mz + i)
          peak.setIntensity(100 - 25*abs(i-2.5) )
          spectrum.push_back(peak)
        exp.addSpectrum(spectrum)

    # Iterate over spectra
    for s in exp:
        for p in s:
            print (s.getRT(), p.getMZ(), p.getIntensity())

    # Sum intensity of all spectra between RT 2.0 and 3.0
    print(sum([p.getIntensity() for s in exp if s.getRT() >= 2.0 and s.getRT() <= 3.0 for p in s]))

    # Store as mzML
    MzMLFile().store("testfile2.mzML", exp)

    ###################################
    # Chromatogram
    # ************
    ###################################

    chromatogram = MSChromatogram()
    rt = range(1500, 500, -100)
    i = [gaussian(rtime, 1000, 150) for rtime in rt]
    chromatogram.set_peaks([rt, i])

    # Sort the peaks according to ascending retention time
    chromatogram.sortByPosition()

    # Iterate over chromatogram of those peaks
    for p in chromatogram:
        print(p.getRT(), p.getIntensity())

    # Access a peak by index
    print(chromatogram[1].getRT(), chromatogram[1].getIntensity())

    chromatogram.setNativeID("Trace XIC@405.2")
    p = Precursor()
    p.setIsolationWindowLowerOffset(1.5)
    p.setIsolationWindowUpperOffset(1.5) 
    p.setMZ(405.2) # isolation at 405.2 +/- 1.5 Th
    p.setActivationEnergy(40) # 40 eV
    p.setCharge(2) # 2+ ion
    p.setMetaValue("description", "test_description")
    p.setMetaValue("description", chromatogram.getNativeID())
    p.setMetaValue("peptide_sequence", chromatogram.getNativeID())
    chromatogram.setPrecursor(p)
    p = Product()
    p.setMZ(603.4) # transition from 405.2 -> 603.4
    chromatogram.setProduct(p)

    # Store as mzML
    exp = MSExperiment()
    exp.addChromatogram(chromatogram)
    MzMLFile().store("testfile3.mzML", exp)




from Types cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.string cimport string as libcpp_string
from StringList cimport *
from MSSpectrum cimport *


cdef extern from "<OpenMS/CHEMISTRY/Tagger.h>" namespace "OpenMS":

    cdef cppclass Tagger:

        Tagger(Tagger) nogil except +

        Tagger(size_t min_tag_length,
               double ppm,
               size_t max_tag_length,
               size_t min_charge,
               size_t max_charge,
               const StringList& fixed_mods,
               const StringList& var_mods) nogil except +

        void getTag(const libcpp_vector[ double ]& mzs,
                    libcpp_vector[ libcpp_string ]& tags) nogil except +

        void getTag(const MSSpectrum& spec,
                    libcpp_vector[ libcpp_string ]& tags) nogil except +

        void setMaxCharge(size_t max_charge) nogil except +



from Types cimport *
from CVTermList cimport *
from Peak1D cimport *
from Map cimport *

cdef extern from "<OpenMS/METADATA/Precursor.h>" namespace "OpenMS":

    cdef cppclass Precursor(Peak1D, CVTermList):
        # wrap-inherits:
        #    Peak1D
        #    CVTermList

        Precursor() nogil except +
        Precursor(Precursor) nogil except +

        # returns a mutable reference to the activation methods
        libcpp_set[ActivationMethod] getActivationMethods() nogil except +
        # sets the activation methods
        void setActivationMethods(libcpp_set[ActivationMethod] activation_methods) nogil except +

        double getActivationEnergy() nogil except +
        void setActivationEnergy(double activation_energy) nogil except +

        double getIsolationWindowLowerOffset() nogil except +
        void setIsolationWindowLowerOffset(double bound) nogil except +

        double getDriftTime() nogil except +
        void setDriftTime(double drift_time) nogil except +

        double getIsolationWindowUpperOffset() nogil except +
        void setIsolationWindowUpperOffset(double bound) nogil except +

        double getDriftTimeWindowLowerOffset() nogil except +
        void setDriftTimeWindowLowerOffset(double drift_time) nogil except +
        double getDriftTimeWindowUpperOffset() nogil except +
        void setDriftTimeWindowUpperOffset(double drift_time) nogil except +

        int getCharge() nogil except +
        void setCharge(int charge) nogil except +

        #Non-mutable access to possible charge states
        libcpp_vector[int] getPossibleChargeStates() nogil except +
        #Sets the possible charge states
        void setPossibleChargeStates(libcpp_vector[int] possible_charge_states) nogil except +

        # Returns the uncharged mass of the precursor, if charge is unknown, i.e. 0 best guess is its doubly charged
        double getUnchargedMass() nogil except +

        bool operator==(Precursor)  nogil except +
        bool operator!=(Precursor)  nogil except +

cdef extern from "<OpenMS/METADATA/Precursor.h>" namespace "OpenMS::Precursor":
    cdef enum ActivationMethod:
      CID,                      #< Collision-induced dissociation
      PSD,                      #< Post-source decay
      PD,                       #< Plasma desorption
      SID,                      #< Surface-induced dissociation
      BIRD,                     #< Blackbody infrared radiative dissociation
      ECD,                      #< Electron capture dissociation
      IMD,                      #< Infrared multiphoton dissociation
      SORI,                     #< Sustained off-resonance irradiation
      HCID,                     #< High-energy collision-induced dissociation
      LCID,                     #< Low-energy collision-induced dissociation
      PHD,                      #< Photodissociation
      ETD,                      #< Electron transfer dissociation
      PQD,                      #< Pulsed q dissociation
      SIZE_OF_ACTIVATIONMETHOD




from MSExperiment  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from PeakFileOptions cimport *
from IMSDataConsumer cimport *

cdef extern from "<OpenMS/FORMAT/MzXMLFile.h>" namespace "OpenMS":

    cdef cppclass MzXMLFile(ProgressLogger):
        # wrap-inherits:
        #   ProgressLogger

        MzXMLFile() nogil except +

        void load(String, MSExperiment &) nogil except+
        void store(String, MSExperiment &) nogil except+

        void transform(String, IMSDataConsumer[Peak1D, ChromatogramPeak] *) nogil except + # wrap-ignore

        PeakFileOptions getOptions() nogil except +
        void setOptions(PeakFileOptions) nogil except +



from Types cimport *
from libcpp cimport bool
from String cimport *
from MSSpectrum cimport *
from ConsensusMap cimport *
from FeatureMap cimport *

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/DataFilters.h>" namespace "OpenMS":

    cdef cppclass DataFilters "OpenMS::DataFilters":
        DataFilters() nogil except +
        DataFilters(DataFilters) nogil except + #wrap-ignore
        Size size() nogil except +
        DataFilter operator[](size_t) nogil except + # wrap-upper-limit:size()
        void add(DataFilter & filter_) nogil except +
        void remove(Size index) nogil except +
        void replace(Size index, DataFilter & filter_) nogil except +
        void clear() nogil except +
        void setActive(bool is_active) nogil except +
        bool isActive() nogil except +
        bool passes(Feature & feature) nogil except +
        bool passes(ConsensusFeature & consensus_feature) nogil except +
        bool passes(MSSpectrum & spectrum, Size peak_index) nogil except +

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/DataFilters.h>" namespace "OpenMS::DataFilters":

    cdef cppclass DataFilter "OpenMS::DataFilters::DataFilter":
        DataFilter() nogil except +
        DataFilter(DataFilter) nogil except + #wrap-ignore
        FilterType field
        FilterOperation op
        double value
        String value_string
        String meta_name
        bool value_is_numerical
        String toString() nogil except +
        void fromString(const String & filter_) nogil except +
        bool operator==(DataFilter & rhs) nogil except +
        bool operator!=(DataFilter & rhs) nogil except +


cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/DataFilters.h>" namespace "OpenMS::DataFilters":
    cdef enum FilterType "OpenMS::DataFilters::FilterType":
        #wrap-attach:
        #    DataFilters
        INTENSITY
        QUALITY
        CHARGE
        SIZE
        META_DATA

cdef extern from "<OpenMS/FILTERING/DATAREDUCTION/DataFilters.h>" namespace "OpenMS::DataFilters":
    cdef enum FilterOperation "OpenMS::DataFilters::FilterOperation":
        #wrap-attach:
        #    DataFilters
        GREATER_EQUAL
        EQUAL
        LESS_EQUAL
        EXISTS




cdef extern from *:
    cdef T dynamic_cast[T](void *) except +
cdef extern from "boost/smart_ptr/shared_ptr.hpp" namespace "boost":
    cdef shared_ptr[T] dynamic_pointer_cast[T,U](shared_ptr[U]& r) except +
from SpectrumAccessOpenMS cimport SpectrumAccessOpenMS as _SpectrumAccessOpenMS
from SpectrumAccessOpenMSCached cimport SpectrumAccessOpenMSCached as _SpectrumAccessOpenMSCached
from SpectrumAccessOpenMSInMemory cimport SpectrumAccessOpenMSInMemory as _SpectrumAccessOpenMSInMemory
from SpectrumAccessQuadMZTransforming cimport SpectrumAccessQuadMZTransforming as _SpectrumAccessQuadMZTransforming
ctypedef _SpectrumAccessOpenMS* _SpectrumAccessOpenMSPtr
ctypedef _SpectrumAccessOpenMSCached* _SpectrumAccessOpenMSCachedPtr
ctypedef _SpectrumAccessOpenMSInMemory* _SpectrumAccessOpenMSInMemoryPtr
ctypedef _SpectrumAccessQuadMZTransforming * _SpectrumAccessQuadMZTransformingPtr


    def getSpectrumPtr(self):
        _r = self.inst.get().sptr

        if (_r.get() == NULL):
          return None

        cdef _SpectrumAccessOpenMS * ptr_sa = dynamic_cast[ _SpectrumAccessOpenMSPtr ](_r.get() )
        cdef _SpectrumAccessOpenMSInMemory * ptr_inmem = dynamic_cast[ _SpectrumAccessOpenMSInMemoryPtr ](_r.get() )
        cdef _SpectrumAccessOpenMSCached * ptr_cached = dynamic_cast[ _SpectrumAccessOpenMSCachedPtr ](_r.get() )
        cdef _SpectrumAccessQuadMZTransforming * ptr_quad = dynamic_cast[ _SpectrumAccessQuadMZTransformingPtr ](_r.get() )

        if (ptr_sa != NULL):
          res_sa = SpectrumAccessOpenMS(__createUnsafeObject__=True)
          res_sa.inst = dynamic_pointer_cast[_SpectrumAccessOpenMS, _ISpectrumAccess](_r)
          return res_sa
        elif (ptr_inmem != NULL):
          res_inmem = SpectrumAccessOpenMSInMemory(__createUnsafeObject__=True)
          res_inmem.inst = dynamic_pointer_cast[_SpectrumAccessOpenMSInMemory, _ISpectrumAccess](_r)
          return res_inmem
        elif (ptr_cached != NULL):
          res_cached = SpectrumAccessOpenMSCached(__createUnsafeObject__=True)
          res_cached.inst = dynamic_pointer_cast[_SpectrumAccessOpenMSCached, _ISpectrumAccess](_r)
          return res_cached
        elif (ptr_quad != NULL):
          res_quad = SpectrumAccessQuadMZTransforming(__createUnsafeObject__=True)
          res_quad.inst = dynamic_pointer_cast[_SpectrumAccessQuadMZTransforming, _ISpectrumAccess](_r)
          return res_quad
        else:
          raise Exception("Did not find suitable conversion to Python object")

    def setSpectrumPtr(self, arg):
        cdef SpectrumAccessOpenMS arg_sa 
        cdef SpectrumAccessOpenMSCached arg_cached 
        cdef SpectrumAccessOpenMSInMemory arg_inmem 
        cdef SpectrumAccessQuadMZTransforming arg_quad 
        if isinstance(arg, SpectrumAccessOpenMS):
            arg_sa = arg
            self.inst.get().sptr = dynamic_pointer_cast[_ISpectrumAccess,_SpectrumAccessOpenMS](arg_sa.inst)
        elif isinstance(arg, SpectrumAccessOpenMSCached):
            arg_cached = arg
            self.inst.get().sptr = dynamic_pointer_cast[_ISpectrumAccess,_SpectrumAccessOpenMSCached](arg_cached.inst)
        elif isinstance(arg, SpectrumAccessOpenMSInMemory):
            arg_inmem = arg
            self.inst.get().sptr = dynamic_pointer_cast[_ISpectrumAccess,_SpectrumAccessOpenMSInMemory](arg_inmem.inst)
        elif isinstance(arg, SpectrumAccessQuadMZTransforming):
            arg_quad = arg
            self.inst.get().sptr = dynamic_pointer_cast[_ISpectrumAccess,_SpectrumAccessQuadMZTransforming](arg_quad.inst)
        else:
          raise Exception("Need to provide suitable ISpectrumAccess-derived child class")




from Types cimport *

from SpectrumMetaDataLookup cimport *
from ProteinIdentification cimport *
from PeptideIdentification cimport *
from MSExperiment cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/FORMAT/PepXMLFile.h>" namespace "OpenMS":

    cdef cppclass PepXMLFile:

        PepXMLFile() nogil except +

        # Since PepXML may not store the complete information, it may be
        # necessary to also pass a lookup structure from which retention times
        # can be extracted.

        void load(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids
                  ) nogil except +

        void load(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  String experiment_name
                  ) nogil except +

        void load(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  String experiment_name,
                  SpectrumMetaDataLookup lookup
                  ) nogil except +

        void store(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids
                  ) nogil except +

        void store(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  String mz_file,
                  String mz_name,
                  bool peptideprophet_analyzed,
                  double rt_tolerance
                  ) nogil except +

        void keepNativeSpectrumName(bool keep) nogil except +




from Types cimport *
from String cimport *
from RANSACModelLinear cimport *
from RANSACModelQuadratic cimport *

cdef extern from "<OpenMS/MATH/MISC/RANSAC.h>" namespace "OpenMS::Math":

    cdef cppclass RANSAC[TModelType]:
        # wrap-instances:
        #   RANSAC := RANSAC[RansacModelLinear]
        #   RANSACQuadratic := RANSAC[RansacModelQuadratic]

        RANSAC() nogil except +
        RANSAC(RANSAC[TModelType] &) nogil except + # wrap-ignore

    cdef cppclass RANSACParam:

        RANSACParam() nogil except +
        RANSACParam(RANSACParam) nogil except + # wrap-ignore
        RANSACParam(size_t p_n, size_t p_k, double p_t, size_t p_d, bool p_relative_d) nogil except +

        String toString() nogil except +

        size_t n #; //< data points; The minimum number of data points required to fit the model
        size_t k # ; //< iterations; The maximum number of iterations allowed in the algorithm 
        double t # ; //< Threshold value; for determining when a data point fits a model. Corresponds to the maximal squared deviation in units of the _second_ dimension (dim2).
        size_t d # ; //< The number of close data values (according to 't') required to assert that a model fits well to data
        bool relative_d #; //< Should 'd' be interpreted as percentages (0-100) of data input size.
        # int (*rng)(int); //< Optional RNG function (useful for testing with fixed seeds)

cdef extern from "<OpenMS/MATH/MISC/RANSAC.h>" namespace "OpenMS::Math::RANSAC<OpenMS::Math::RansacModelLinear>":

    libcpp_vector[libcpp_pair[double,double]] ransac(
            libcpp_vector[libcpp_pair[double,double]] pairs,
            size_t n, size_t k, double t, size_t d, bool test
            ) nogil except + # wrap-attach:RANSAC

cdef extern from "<OpenMS/MATH/MISC/RANSAC.h>" namespace "OpenMS::Math::RANSAC<OpenMS:Math::RansacModelQuadratic>":

    libcpp_vector[libcpp_pair[double,double]] ransac(
            libcpp_vector[libcpp_pair[double,double]] pairs,
            size_t n, size_t k, double t, size_t d, bool test
            ) nogil except + # wrap-attach:RANSACQuadratic




from String cimport *
from Types cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/DateTime.h>" namespace "OpenMS":

    cdef cppclass DateTime:

        DateTime()   nogil except +
        DateTime(DateTime) nogil except + # wrap-ignore

        void setDate(String date) nogil except +

        void setTime(String date) nogil except +

        # void setDate(UInt month, UInt day, UInt year) nogil except +

        # void setTime(UInt hour, UInt minute, UInt second) nogil except +

        # void set(UInt month, UInt day, UInt year, UInt hour, UInt minute, UInt second) nogil except +

        # void get(UInt month, UInt day, UInt year, UInt hour, UInt minute, UInt second) nogil except +

        # void getDate(UInt month, UInt day, UInt year) nogil except +

        String getDate() nogil except +

        # void getTime(UInt hour, UInt minute, UInt second) nogil except +

        String getTime() nogil except +

        # Returns the current date and time
        DateTime now() nogil except +

        #Sets the undefined date: 00/00/0000 00:00:00
        void clear() nogil except +

        #   @brief Returns a string representation of the date and time
        #   The format of the string will be yyyy-MM-dd hh:mm:ss
        String get() nogil except +

        #     @brief Sets date and time
        #     The following formats are supported:
        #     - MM/dd/yyyy hh:mm:ss
        #     - dd.MM.yyyy hh:mm:ss
        #     - yyyy-MM-dd hh:mm:ss
        #     - yyyy-MM-ddThh:mm:ss (ISO 8601 format)
        #     - yyyy-MM-ddZ (ISO 8601 format)
        #     - yyyy-MM-dd+hh:mm (ISO 8601 format)
        void set(String date) nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/DateTime.h>" namespace "OpenMS::DateTime":

    DateTime now() # wrap-attach:DateTime







from Types cimport *
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from Peak1D cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/FilterFunctor.h>" namespace "OpenMS":
    
    cdef cppclass FilterFunctor(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler
        FilterFunctor() nogil except +
        FilterFunctor(FilterFunctor) nogil except +
        # double apply(MSSpectrum & ) nogil except +
        void registerChildren() nogil except +




import pyopenms
import os.path

from .collections_ import Counter

def test0():
    fh = pyopenms.MzMLFile()
    here = os.path.dirname(os.path.abspath(__file__))
    path = os.path.join(here, "test2.mzML").encode()

    class Consumer(object):

        def __init__(self):
            self.speclevels = []
            self.rts = []

        def consumeSpectrum(self, spec):
            self.speclevels.append(spec.getMSLevel())
            self.rts.append(spec.getRT())

        def consumeChromatogram(self, chromo):
            raise Exception("should never be called as we have no chromoatograms in example file")

        def setExpectedSize(self, num_specs, num_chromo):
            assert num_specs == 5, num_specs
            assert num_chromo == 0, num_chromo

        def setExperimentalSettings(self, exp):
            assert isinstance(exp, pyopenms.ExperimentalSettings)

    consumer = Consumer()
    fh.transform(path, consumer)
    cc = Counter(consumer.speclevels)
    assert set(cc.keys()) == set([1, 2])
    assert cc[1] == 2
    assert cc[2] == 3
    assert abs(min(consumer.rts) - 4200.76) < 0.01
    assert abs(max(consumer.rts) - 4202.03) < 0.01



from libcpp cimport bool
from Types cimport *
from String cimport *

cdef extern from "<OpenMS/SYSTEM/BuildInfo.h>" namespace "OpenMS::Internal":

    cdef cppclass OpenMSOSInfo:

        OpenMSOSInfo() nogil except +
        OpenMSOSInfo(OpenMSOSInfo) nogil except + # wrap-ignore
        String getOSAsString() nogil except +
        String getArchAsString() nogil except +
        String getOSVersionAsString() nogil except +

    cdef cppclass OpenMSBuildInfo:

        OpenMSBuildInfo() nogil except +
        OpenMSBuildInfo(OpenMSBuildInfo) nogil except + # wrap-ignore


cdef extern from "<OpenMS/SYSTEM/BuildInfo.h>" namespace "OpenMS::Internal::OpenMSOSInfo":

    OpenMSOSInfo getOSInfo() nogil except + # wrap-attach:OpenMSOSInfo

    String getBinaryArchitecture() nogil except + # wrap-attach:OpenMSOSInfo


cdef extern from "<OpenMS/SYSTEM/BuildInfo.h>" namespace "OpenMS::Internal::OpenMSBuildInfo":

    bool isOpenMPEnabled() nogil except + # wrap-attach:OpenMSBuildInfo

    String getBuildType() nogil except + # wrap-attach:OpenMSBuildInfo

    Size getOpenMPMaxNumThreads() nogil except + # wrap-attach:OpenMSBuildInfo



from Types cimport *
from String cimport *
from ProgressLogger cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/FORMAT/MS2File.h>" namespace "OpenMS":
    
    cdef cppclass MS2File(ProgressLogger) :
        # wrap-inherits:
        #  ProgressLogger
        MS2File() nogil except +
        MS2File(MS2File) nogil except + #wrap-ignore
        void load(const String & filename, MSExperiment & exp) nogil except +




from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/FILTERING/BASELINE/MorphologicalFilter.h>" namespace "OpenMS":

    cdef cppclass MorphologicalFilter(DefaultParamHandler,ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        MorphologicalFilter()      nogil except +
        # MorphologicalFilter(MorphologicalFilter)      nogil except + #private

        void filter(MSSpectrum & spectrum)      nogil except +
        void filterExperiment(MSExperiment & exp)      nogil except +




from Types cimport *
from String cimport *
from FeatureMap cimport *
from MSExperiment cimport *
from FeatureMapping cimport * 
from KDTreeFeatureMaps cimport *
from libcpp.vector cimport vector as libcpp_vector
from libcpp.pair cimport pair as libcpp_pair

cdef extern from "<OpenMS/ANALYSIS/ID/SiriusAdapterAlgorithm.h>" namespace "OpenMS":

    cdef cppclass SiriusAdapterAlgorithm(DefaultParamHandler):

        SiriusAdapterAlgorithm() nogil except +
        SiriusAdapterAlgorithm(SiriusAdapterAlgorithm) nogil except +

        String getFeatureOnly() nogil except +
        String getNoMasstraceInfoIsotopePattern() nogil except +
        Int getIsotopePatternIterations() nogil except +
        Int getCandidates() nogil except +
        Int getTopNHits() nogil except +

        void preprocessingSirius(String featureinfo,
                                 MSExperiment& spectra,                
                                 libcpp_vector[FeatureMap]& v_fp,
                                 KDTreeFeatureMaps& fp_map_kd,
                                 SiriusAdapterAlgorithm sirius_algo,
                                 FeatureMapping_FeatureToMs2Indices& feature_mapping)

        void checkFeatureSpectraNumber(String featureinfo,
                                       FeatureMapping_FeatureToMs2Indices feature_mapping,
                                       MSExperiment spectra, 
                                       SiriusAdapterAlgorithm sirius_algo);

        libcpp_vector[String] callSiriusQProcess(String tmp_ms_file,
                                                 String tmp_out_dir,
                                                 String executable,
                                                 String out_csifingerid,
                                                 SiriusAdapterAlgorithm sirius_algo);

cdef extern from "<OpenMS/ANALYSIS/ID/SiriusAdapterAlgorithm.h>" namespace "OpenMS::SiriusAdapterAlgorithm":
 
    cdef cppclass SiriusTemporaryFileSystemObjects "OpenMS::SiriusAdapterAlgorithm::SiriusTemporaryFileSystemObjects":
        SiriusTemporaryFileSystemObjects(int debug_level)
        SiriusTemporaryFileSystemObjects(SiriusTemporaryFileSystemObjects)
        
        String getTmpDir() nogil except +
        String getTmpOutDir() nogil except +
        String getTmpMsFile() nogil except + 




from libcpp cimport bool
from Types cimport *
from Peak2D cimport *
from MetaInfoInterface cimport *
from UniqueIdInterface cimport *

cdef extern from "<OpenMS/KERNEL/RichPeak2D.h>" namespace "OpenMS":


    cdef cppclass RichPeak2D(Peak2D, UniqueIdInterface, MetaInfoInterface):
        # wrap-inherits:
        #    Peak2D
        #    UniqueIdInterface
        #    MetaInfoInterface

        RichPeak2D() nogil except +
        RichPeak2D(RichPeak2D &) nogil except +
        #RichPeak2D(DPosition2 &, float) nogil except +

        bool operator==(RichPeak2D) nogil except +
        bool operator!=(RichPeak2D) nogil except +




from libcpp.vector cimport vector as libcpp_vector
from libcpp.set cimport set as libcpp_set
from libcpp cimport bool

from String cimport *
from DefaultParamHandler cimport *

from PeptideIdentification cimport *
from ProteinIdentification cimport *
from FASTAFile cimport *
from ProteaseDigestion cimport *

from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/FILTERING/ID/IDFilter.h>" namespace "OpenMS":

    cdef cppclass IDFilter:

        IDFilter()           nogil except +
        IDFilter(IDFilter)   nogil except + # wrap-ignore

        Size countHits(libcpp_vector[PeptideIdentification] identifications) nogil except +
        Size countHits(libcpp_vector[ProteinIdentification] identifications) nogil except +

        bool getBestHit(libcpp_vector[PeptideIdentification] identifications, bool assume_sorted, PeptideHit& best_hit) nogil except +
        bool getBestHit(libcpp_vector[ProteinIdentification] identifications, bool assume_sorted, ProteinHit& best_hit) nogil except +

        void extractPeptideSequences(libcpp_vector[PeptideIdentification]& peptides, libcpp_set[String]& sequences, bool ignore_mods) nogil except +

        void updateHitRanks(libcpp_vector[PeptideIdentification]& identifications) nogil except +
        void updateHitRanks(libcpp_vector[ProteinIdentification]& identifications) nogil except +

        void removeUnreferencedProteins(libcpp_vector[ProteinIdentification]& proteins, libcpp_vector[PeptideIdentification]& peptides) nogil except +

        void updateProteinReferences(libcpp_vector[PeptideIdentification]& peptides, libcpp_vector[ProteinIdentification]& proteins, bool remove_peptides_without_reference) nogil except +

        bool updateProteinGroups(libcpp_vector[ProteinGroup]& groups, libcpp_vector[ProteinHit]& hits) nogil except +

        void removeEmptyIdentifications(libcpp_vector[PeptideIdentification]& ids) nogil except +
        void removeEmptyIdentifications(libcpp_vector[ProteinIdentification]& ids) nogil except +

        void filterHitsByScore(libcpp_vector[PeptideIdentification]& ids, double threshold_score) nogil except +
        void filterHitsByScore(libcpp_vector[ProteinIdentification]& ids, double threshold_score) nogil except +

        void keepNBestHits(libcpp_vector[PeptideIdentification]& ids, Size n) nogil except +
        void keepNBestHits(libcpp_vector[ProteinIdentification]& ids, Size n) nogil except +

        void filterHitsByRank(libcpp_vector[PeptideIdentification]& ids, Size min_rank, Size max_rank) nogil except +
        void filterHitsByRank(libcpp_vector[ProteinIdentification]& ids, Size min_rank, Size max_rank) nogil except +

        void removeDecoyHits(libcpp_vector[PeptideIdentification]& ids) nogil except +
        void removeDecoyHits(libcpp_vector[ProteinIdentification]& ids) nogil except +

        void removeHitsMatchingProteins(libcpp_vector[PeptideIdentification]& ids, libcpp_set[String] accessions) nogil except +
        void removeHitsMatchingProteins(libcpp_vector[ProteinIdentification]& ids, libcpp_set[String] accessions) nogil except +

        void keepHitsMatchingProteins(libcpp_vector[PeptideIdentification]& ids, libcpp_set[String] accessions) nogil except +
        void keepHitsMatchingProteins(libcpp_vector[ProteinIdentification]& ids, libcpp_set[String] accessions) nogil except +

        void keepBestPeptideHits(libcpp_vector[PeptideIdentification]& peptides, bool strict) nogil except +

        void filterPeptidesByLength(libcpp_vector[PeptideIdentification]& peptides, Size min_length, Size max_length) nogil except +

        void filterPeptidesByCharge(libcpp_vector[PeptideIdentification]& peptides, Size min_charge, Size max_charge) nogil except +

        void filterPeptidesByRT(libcpp_vector[PeptideIdentification]& peptides, Size min_rt, Size max_rt) nogil except +

        void filterPeptidesByMZ(libcpp_vector[PeptideIdentification]& peptides, Size min_mz, Size max_mz) nogil except +

        void filterPeptidesByMZError(libcpp_vector[PeptideIdentification]& peptides, double mass_error, bool unit_ppm) nogil except +

        void filterPeptidesByRTPredictPValue(libcpp_vector[PeptideIdentification]& peptides, const String& metavalue_key, double threshold) nogil except +

        void removePeptidesWithMatchingModifications(libcpp_vector[PeptideIdentification]& peptides, libcpp_set[String]& modifications) nogil except +

        void keepPeptidesWithMatchingModifications(libcpp_vector[PeptideIdentification]& peptides, libcpp_set[String]& modifications) nogil except +

        void removePeptidesWithMatchingSequences(libcpp_vector[PeptideIdentification]& peptides, libcpp_vector[PeptideIdentification]& bad_peptides, bool ignore_mods) nogil except +

        void keepPeptidesWithMatchingSequences(libcpp_vector[PeptideIdentification]& peptides, libcpp_vector[PeptideIdentification]& bad_peptides, bool ignore_mods) nogil except +

        void keepUniquePeptidesPerProtein(libcpp_vector[PeptideIdentification]& peptides) nogil except +

        void removeDuplicatePeptideHits(libcpp_vector[PeptideIdentification]& peptides) nogil except +

        void filterHitsByScore(MSExperiment& experiment, double peptide_threshold_score, double protein_threshold_score) nogil except +

        void keepNBestHits(MSExperiment& experiment, Size n) nogil except +

        void keepHitsMatchingProteins(MSExperiment& experiment, libcpp_vector[FASTAEntry]& proteins) nogil except +

cdef extern from "<OpenMS/FILTERING/ID/IDFilter.h>" namespace "OpenMS::IDFilter":
    
    cdef cppclass DigestionFilter "OpenMS::IDFilter::DigestionFilter":
        # wrap-attach:
        #    IDFilter
        DigestionFilter() nogil except + # wrap-ignore
        DigestionFilter(DigestionFilter) nogil except + #wrap-ignore

        # GetMatchingItems[ PeptideEvidence, FASTAEntry ] accession_resolver_
        ProteaseDigestion digestion_
        bool ignore_missed_cleavages_
        bool methionine_cleavage_

        DigestionFilter(libcpp_vector[ FASTAEntry ] & entries, 
                        ProteaseDigestion & digestion,
                        bool ignore_missed_cleavages,
                        bool methionine_cleavage) nogil except +

        # bool operator()(PeptideEvidence & evidence) nogil except +
        void filterPeptideEvidences(libcpp_vector[ PeptideIdentification ] & peptides) nogil except +




from Types cimport *
from libcpp cimport bool
from DefaultParamHandler cimport *
from MSSpectrum cimport *
from AASequence cimport *
from ProteinProteinCrossLink cimport *

cdef extern from "<OpenMS/CHEMISTRY/TheoreticalSpectrumGeneratorXLMS.h>" namespace "OpenMS":

    cdef cppclass TheoreticalSpectrumGeneratorXLMS(DefaultParamHandler) :
        # wrap-inherits:
        #  DefaultParamHandler

        TheoreticalSpectrumGeneratorXLMS() nogil except +
        TheoreticalSpectrumGeneratorXLMS(TheoreticalSpectrumGeneratorXLMS) nogil except +

        void getLinearIonSpectrum(MSSpectrum& spectrum, AASequence peptide,
                Size link_pos, bool frag_alpha, int charge, Size link_pos_2) nogil except +

        void getXLinkIonSpectrum(MSSpectrum& spectrum, AASequence peptide,
                Size link_pos, double precursor_mass, bool frag_alpha,
                int mincharge, int maxcharge, Size link_pos_2) nogil except +

        void getXLinkIonSpectrum(MSSpectrum& spectrum, ProteinProteinCrossLink crosslink,
                bool frag_alpha, int mincharge, int maxcharge) nogil except +



from Types cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/DATAACCESS/SiriusMzTabWriter.h>" namespace "OpenMS::SiriusMzTabWriter":
    
    cdef cppclass SiriusAdapterHit "OpenMS::SiriusMzTabWriter::SiriusAdapterHit":
        SiriusAdapterHit() nogil except +
        SiriusAdapterHit(SiriusAdapterHit) nogil except + #wrap-ignore

        String formula
        String adduct
        int rank
        double score
        double treescore
        double isoscore
        int explainedpeaks
        double explainedintensity




from . import Constants
from . import Plotting as Plot

class SimpleOpenMSSpectraFactory:

    @staticmethod
    def getSpectrumAccessOpenMSPtr(exp):
      is_cached = False

      for i in range(exp.size()):
        for dp in exp[i].getDataProcessing():
          if dp.metaValueExists("cached_data"):
            is_cached = True

      for chrom in exp.getChromatograms():
        for dp in chrom.getDataProcessing():
          if dp.metaValueExists("cached_data"):
            is_cached = True

      if is_cached:
        return SpectrumAccessOpenMSCached( exp.getLoadedFilePath() )
      else:
        return SpectrumAccessOpenMS( exp )





from String cimport *

from ProteinIdentification cimport *
from PeptideIdentification cimport *

cdef extern from "<OpenMS/FORMAT/IdXMLFile.h>" namespace "OpenMS":

    cdef cppclass IdXMLFile:

        IdXMLFile() nogil except +

        void load(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  ) nogil except +

        void store(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  String document_id) nogil except +

        void store(String filename,
                  libcpp_vector[ProteinIdentification] & protein_ids,
                  libcpp_vector[PeptideIdentification] & peptide_ids,
                  ) nogil except +



from ProgressLogger cimport *
from DefaultParamHandler cimport *
from MSExperiment cimport *
from MSSpectrum cimport *
from Peak1D cimport *
from ChromatogramPeak cimport *

cdef extern from "<OpenMS/FILTERING/SMOOTHING/SavitzkyGolayFilter.h>" namespace "OpenMS":

    cdef cppclass SavitzkyGolayFilter(DefaultParamHandler,ProgressLogger):
        # wrap-inherits:
        #    DefaultParamHandler
        #    ProgressLogger

        SavitzkyGolayFilter()      nogil except +
        SavitzkyGolayFilter(SavitzkyGolayFilter)      nogil except +

        void filter(MSSpectrum & spectrum)      nogil except +
        void filterExperiment(MSExperiment & exp)      nogil except +





from String cimport *
from Gradient cimport *

cdef extern from "<OpenMS/METADATA/HPLC.h>" namespace "OpenMS":

    cdef cppclass HPLC:

        HPLC() nogil except +
        HPLC(HPLC) nogil except + # wrap-ignore

        # returns a reference to the instument name
        String getInstrument() nogil except +
        # sets the instument name
        void setInstrument(String instrument) nogil except +

        # returns a reference to the column description
        String getColumn() nogil except +
        # sets the column description
        void setColumn(String column) nogil except +

        # returns the temperature (in degree C)
        Int getTemperature() nogil except +
        # sets the temperature (in degree C)
        void setTemperature(Int temperature) nogil except +

        # returns the pressure (in bar)
        UInt getPressure() nogil except +
        # sets the pressure (in bar)
        void setPressure(UInt pressure) nogil except +

        # returns the flux (in microliter/sec)
        UInt getFlux() nogil except +
        # sets the flux (in microliter/sec)
        void setFlux(UInt flux) nogil except +

        # returns the comments
        String getComment() nogil except +
        # sets the comments
        void setComment(String comment) nogil except +

        # returns a mutable reference to the used gradient
        Gradient getGradient() nogil except +
        # sets the used gradient
        void setGradient(Gradient gradient) nogil except +





from libcpp cimport bool
from String cimport *
from CVTerm cimport *
from CVMappingTerm cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/CVMappingRule.h>" namespace "OpenMS":

    cdef cppclass CVMappingRule:

        CVMappingRule()               nogil except +
        CVMappingRule(CVMappingRule)  nogil except +

        # sets the identifier of the rule
        void setIdentifier(String identifier) nogil except +

        # returns the identifier of the rule
        String getIdentifier() nogil except +

        # sets the path of the element, where this rule is allowed
        void setElementPath(String element_path) nogil except +

        # returns the path of the element, where this rule is allowed
        String getElementPath() nogil except +

        # sets the requirement level of this rule
        void setRequirementLevel(RequirementLevel level) nogil except +

        # returns the requirement level of this rule
        RequirementLevel getRequirementLevel() nogil except +

        # sets the combination operator of the rule
        void setCombinationsLogic(CombinationsLogic combinations_logic) nogil except +

        # returns the combinations operator of the rule
        CombinationsLogic getCombinationsLogic() nogil except +

        # sets the scope path of the rule
        void setScopePath(String path) nogil except +

        # returns the scope path of the rule
        String getScopePath() nogil except +

        # sets the terms which are allowed
        void setCVTerms(libcpp_vector[CVMappingTerm] cv_terms) nogil except +

        # returns the allowed terms
        libcpp_vector[CVMappingTerm] getCVTerms() nogil except +

        # adds a term to the allowed terms
        void addCVTerm(CVMappingTerm cv_terms) nogil except +

        # equality operator
        bool operator==(CVMappingRule rhs) nogil except +

        # inequality operator
        bool operator!=(CVMappingRule rhs) nogil except +

cdef extern from "<OpenMS/DATASTRUCTURES/CVMappingRule.h>" namespace "OpenMS::CVMappingRule":

    # enum to specify the requirement level
    cdef enum RequirementLevel:
        # wrap-attach:
        #     CVMappingRule
        MUST = 0,
        SHOULD = 1,
        MAY = 2

    # enum to specify the combination operator
    cdef enum CombinationsLogic:
        # wrap-attach:
        #     CVMappingRule
        OR = 0,
        AND = 1,
        XOR = 2



from DPosition cimport *

cdef extern from "<OpenMS/DATASTRUCTURES/DBoundingBox.h>" namespace "OpenMS":

    # this is the way to declare a class with a int template parameter:
    # as an alternative one could use a similar ctypedef, but these can
    # not be renamed when cipmorting, which might cause trouble !

    cdef cppclass DBoundingBox2 "OpenMS::DBoundingBox<2> ":
        DBoundingBox2() nogil except +
        DBoundingBox2(DBoundingBox2) nogil except +
        DPosition2 minPosition() nogil except +
        DPosition2 maxPosition() nogil except +







from libcpp.vector cimport vector as libcpp_vector
from libcpp.map cimport map as libcpp_map
from String cimport *
from Feature cimport *
from OpenSwathScoring cimport *

cdef extern from "<OpenMS/KERNEL/MRMFeature.h>" namespace "OpenMS":

    cdef cppclass MRMFeature(Feature):
        #
        # wrap-inherits:
        #    Feature

        MRMFeature() nogil except +
        MRMFeature(MRMFeature &) nogil except +

        OpenSwath_Scores getScores() nogil except +
        void setScores(OpenSwath_Scores s) nogil except +

        Feature getFeature(String key) nogil except +
        void addFeature(Feature & f, String key) nogil except +
        libcpp_vector[Feature] getFeatures() nogil except +
        void getFeatureIDs(libcpp_vector[String] & result) nogil except +

        Feature getPrecursorFeature(String key) nogil except +
        void addPrecursorFeature(Feature & f, String key) nogil except +
        void getPrecursorFeatureIDs(libcpp_vector[String] & result) nogil except +

        bool operator==(MRMFeature) nogil except +
        bool operator!=(MRMFeature) nogil except +




from DefaultParamHandler cimport *
from MSExperiment cimport *
from Param cimport *
from String cimport *

cdef extern from "<OpenMS/FORMAT/MSPGenericFile.h>" namespace "OpenMS":
    cdef cppclass MSPGenericFile(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler

        MSPGenericFile() nogil except +
        MSPGenericFile(MSPGenericFile) nogil except +
        MSPGenericFile(const String& filename, MSExperiment& library) nogil except +

        void load(const String& filename, MSExperiment& library) nogil except +
        void getDefaultParameters(Param & params) nogil except +




import argparse
import pyopenms as pms
from common import addDataProcessing, writeParamsIfRequested, updateDefaults


def run_featurefinder_centroided(input_path, params, seeds, out_path):

    fh = pms.MzMLFile()
    options = pms.PeakFileOptions()
    options.setMSLevels([1,1])
    fh.setOptions(options)
    input_map = pms.MSExperiment()
    fh.load(input_path, input_map)
    input_map.updateRanges()

    ff = pms.FeatureFinder()
    ff.setLogType(pms.LogType.CMD)

    features = pms.FeatureMap()
    name = pms.FeatureFinderAlgorithmPicked.getProductName()
    ff.run(name, input_map, features, params, seeds)

    features.setUniqueIds()
    addDataProcessing(features, params, pms.ProcessingAction.QUANTITATION)

    fh = pms.FeatureXMLFile()
    fh.store(out_path, features)


def main():

    parser = argparse.ArgumentParser(description="FeatureFinderCentroided")
    parser.add_argument("-in",
                        action="store",
                        type=str,
                        dest="in_",
                        metavar="input_file",
                        )

    parser.add_argument("-seeds",
                        action="store",
                        type=str,
                        metavar="seeds_file",
                        )

    parser.add_argument("-out",
                        action="store",
                        type=str,
                        metavar="output_file",
                        )

    parser.add_argument("-ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    parser.add_argument("-write_ini",
                        action="store",
                        type=str,
                        metavar="ini_file",
                        )

    parser.add_argument("-write_dict_ini",
                        action="store",
                        type=str,
                        metavar="python_dict_ini_file",
                        )

    args = parser.parse_args()

    run_mode = args.in_ is not None and args.out is not None\
                and (args.ini is not None or args.dict_ini is not None)
    write_mode = args.write_ini is not None or args.write_dict_ini is not None
    ok = run_mode or write_mode
    if not ok:
        parser.error("either specify -in, -out and -(dict)ini for running "
                     "the peakpicker\nor -write(dict)ini for creating std "
                     "ini file")

    name = pms.FeatureFinderAlgorithmPicked.getProductName()
    defaults = pms.FeatureFinder().getParameters(name)

    write_requested = writeParamsIfRequested(args, defaults)

    if not write_requested:
        updateDefaults(args, defaults)

        seeds = pms.FeatureMap()
        if args.seeds:
            fh = pms.FeatureXMLFile()
            fh.load(args.seeds, seeds)

        run_featurefinder_centroided(args.in_, defaults, seeds, args.out)

if __name__ == "__main__":
    main()





    def getTimeArray(self):
        cdef shared_ptr[_BinaryDataArray] _r = self.inst.get().getTimeArray()
        cdef libcpp_vector[double] _vec = _r.get().data
        cdef list py_result = _vec
        return py_result

    def getIntensityArray(self):
        cdef shared_ptr[_BinaryDataArray] _r = self.inst.get().getIntensityArray()
        cdef libcpp_vector[double] _vec = _r.get().data
        cdef list py_result = _vec
        return py_result

    def setTimeArray(self, list data):
        assert isinstance(data, list), 'arg transitions wrong type'

        cdef shared_ptr[_BinaryDataArray] v0 = shared_ptr[_BinaryDataArray](new _BinaryDataArray() ) 
        cdef libcpp_vector[double] _vec = data
        v0.get().data = data
        self.inst.get().setTimeArray(v0)

    def setIntensityArray(self, list data):
        assert isinstance(data, list), 'arg transitions wrong type'

        cdef shared_ptr[_BinaryDataArray] v0 = shared_ptr[_BinaryDataArray](new _BinaryDataArray() ) 
        cdef libcpp_vector[double] _vec = data
        v0.get().data = data
        self.inst.get().setIntensityArray(v0)




from MSChromatogram cimport *
from MSSpectrum cimport *

cdef extern from "OpenMS/KERNEL/SpectrumHelper.h":

    ## Namespace only
    cdef cppclass SpectrumHelper:
        # wrap-manual-memory
        SpectrumHelper() # wrap-ignore

    void removePeaks(MSChromatogram& p, double pos_start, double pos_end) # wrap-attach:SpectrumHelper
    void removePeaks(MSSpectrum& p, double pos_start, double pos_end) # wrap-attach:SpectrumHelper
    void subtractMinimumIntensity(MSChromatogram& p) # wrap-attach:SpectrumHelper
    void subtractMinimumIntensity(MSSpectrum& p) # wrap-attach:SpectrumHelper



from Types cimport *
from String cimport *
from OpenSwathDataStructures cimport *
from ISpectrumAccess cimport *

cdef extern from "<OpenMS/ANALYSIS/OPENSWATH/DATAACCESS/SpectrumAccessOpenMSCached.h>" namespace "OpenMS":

  # TODO missing functions
  cdef cppclass SpectrumAccessOpenMSCached(ISpectrumAccess):
        # wrap-inherits:
        #  ISpectrumAccess

        SpectrumAccessOpenMSCached() # wrap-pass-constructor

        SpectrumAccessOpenMSCached(String filename) nogil except +
        SpectrumAccessOpenMSCached(SpectrumAccessOpenMSCached q) nogil except + # wrap-ignore




from libcpp cimport bool
from libcpp.vector cimport vector as libcpp_vector
from DataValue cimport *
from String cimport *
from Types cimport *
from MetaInfoInterface cimport *
from DataProcessing cimport *

cdef extern from "<OpenMS/METADATA/MetaInfoDescription.h>" namespace "OpenMS":

    cdef cppclass MetaInfoDescription(MetaInfoInterface):
        # wrap-inherits:
        #   MetaInfoInterface

        MetaInfoDescription() nogil except +
        MetaInfoDescription(MetaInfoDescription) nogil except +

        bool operator==(MetaInfoDescription) nogil except +
        bool operator!=(MetaInfoDescription) nogil except +

        String getName() nogil except +
        void setName(String name) nogil except +

        libcpp_vector[ shared_ptr[DataProcessing] ] getDataProcessing() nogil except +
        void setDataProcessing(libcpp_vector[ shared_ptr[DataProcessing] ]) nogil except +




from Types cimport *

cdef extern from "<OpenMS/METADATA/Tagging.h>" namespace "OpenMS":
    
    cdef cppclass Tagging:

        Tagging() nogil except + 
        Tagging(Tagging) nogil except + 

        double getMassShift() nogil except +
        void setMassShift(double mass_shift) nogil except +

        IsotopeVariant getVariant() nogil except +
        void setVariant(IsotopeVariant variant) nogil except +

cdef extern from "<OpenMS/METADATA/Tagging.h>" namespace "OpenMS::Tagging":
    
    cdef enum IsotopeVariant "OpenMS::Tagging::IsotopeVariant":
        LIGHT
        HEAVY
        SIZE_OF_ISOTOPEVARIANT




from MSChromatogram cimport *
from MSSpectrum cimport *
from DefaultParamHandler cimport *

cdef extern from "<OpenMS/MATH/MISC/EmgGradientDescent.h>" namespace "OpenMS":

    cdef cppclass EmgGradientDescent(DefaultParamHandler):
        # wrap-inherits:
        #  DefaultParamHandler

        EmgGradientDescent() nogil except +
        EmgGradientDescent(EmgGradientDescent) nogil except +

        void getDefaultParameters(Param&) nogil except +

        void fitEMGPeakModel(MSChromatogram& input_peak, MSChromatogram& output_peak) nogil except +
        void fitEMGPeakModel(MSSpectrum& input_peak, MSSpectrum& output_peak) nogil except +
        void fitEMGPeakModel(MSChromatogram& input_peak, MSChromatogram& output_peak, double left_pos, double right_pos) nogil except +
        void fitEMGPeakModel(MSSpectrum& input_peak, MSSpectrum& output_peak, double left_pos, double right_pos) nogil except +



from libcpp cimport bool

# this class has addons, see the ./addons folder

cdef extern from "<OpenMS/DATASTRUCTURES/Map.h>" namespace "OpenMS":

    cdef cppclass Map[U, V]:
        # we have converters for this, do not wrap the class itself !
        # wrap-ignore

        U first
        V second

        Map()
        Map(Map[U,V])

        V& operator[](U&)  nogil

        cppclass iterator:
            bool operator!=(Map.iterator) nogil
            Map operator*() nogil
            Map operator++() nogil

        Map.iterator begin() nogil
        Map.iterator end() nogil




from Types cimport *
from DPosition cimport *
from libcpp cimport bool

cdef extern from "<OpenMS/KERNEL/RangeManager.h>" namespace "OpenMS":
    
    cdef cppclass RangeManager1 "OpenMS::RangeManager<1>":
        # wrap-ignore
        # no-pxd-import
        RangeManager1() nogil except +
        RangeManager1(RangeManager1) nogil except +
        DPosition1 getMin() nogil except +
        DPosition1 getMax() nogil except +
        double getMinInt() nogil except +
        double getMaxInt() nogil except +
        void clearRanges() nogil except +

    cdef cppclass RangeManager2 "OpenMS::RangeManager<2>":
        # wrap-ignore
        # no-pxd-import
        RangeManager2() nogil except +
        RangeManager2(RangeManager2) nogil except +
        DPosition2 getMin() nogil except +
        DPosition2 getMax() nogil except +
        double getMinInt() nogil except +
        double getMaxInt() nogil except +
        void clearRanges() nogil except +



from Types cimport *
from libcpp cimport bool
from libcpp.map cimport map as libcpp_map
from PeakMarker cimport *

cdef extern from "<OpenMS/FILTERING/TRANSFORMERS/NeutralLossMarker.h>" namespace "OpenMS":
    
    cdef cppclass NeutralLossMarker(PeakMarker) :
        # wrap-inherits:
        #  PeakMarker
        NeutralLossMarker() nogil except +
        NeutralLossMarker(NeutralLossMarker) nogil except +
        void apply(libcpp_map[ double, bool ] & , MSSpectrum & ) nogil except +
        PeakMarker * create() nogil except + # wrap-ignore
        # String getProductName() nogil except +




from Types cimport *
from MSSpectrum cimport *
from MSExperiment cimport *

cdef extern from "<OpenMS/ANALYSIS/RNPXL/PScore.h>" namespace "OpenMS":
    
    cdef cppclass PScore "OpenMS::PScore":
        PScore() nogil except + 
        PScore(PScore) nogil except + #wrap-ignore

        libcpp_vector[ size_t ] calculateIntensityRankInMZWindow(libcpp_vector[ double ] & mz, libcpp_vector[ double ] & intensities, double mz_window) nogil except +

        libcpp_vector[ libcpp_vector[ size_t ] ] calculateRankMap(MSExperiment & peak_map, double mz_window) nogil except +

        libcpp_map[ size_t, MSSpectrum ] calculatePeakLevelSpectra(MSSpectrum & spec, libcpp_vector[ size_t ] & ranks, Size min_level, Size max_level) nogil except +

        double computePScore(double fragment_mass_tolerance,
                             bool fragment_mass_tolerance_unit_ppm,
                             libcpp_map[ size_t, MSSpectrum ] & peak_level_spectra,
                             libcpp_vector[ MSSpectrum ] & theo_spectra,
                             double mz_window) nogil except +

        double computePScore(double fragment_mass_tolerance,
                             bool fragment_mass_tolerance_unit_ppm,
                             libcpp_map[ size_t, MSSpectrum ] & peak_level_spectra,
                             MSSpectrum & theo_spectrum,
                             double mz_window) nogil except +

## wrap static methods
# cdef extern from "<OpenMS/ANALYSIS/RNPXL/PScore.h>" namespace "OpenMS::PScore":
# 
#     double massCorrectionTerm(double mass) nogil except + #wrap-attach:PScore
#     double cleavageCorrectionTerm(Size cleavages, bool consecutive_cleavage) nogil except + #wrap-attach:PScore
#     double modificationCorrectionTerm(Size modifications) nogil except + #wrap-attach:PScore
# 



from Types cimport *
from Matrix cimport *

cdef extern from "<OpenMS/MATH/MISC/NonNegativeLeastSquaresSolver.h>" namespace "OpenMS":
    
    cdef cppclass NonNegativeLeastSquaresSolver "OpenMS::NonNegativeLeastSquaresSolver":
        NonNegativeLeastSquaresSolver(NonNegativeLeastSquaresSolver) nogil except + #wrap-ignore
        Int solve(Matrix[ double ] & A, Matrix[ double ] & b, Matrix[ double ] & x) nogil except +

cdef extern from "<OpenMS/MATH/MISC/NonNegativeLeastSquaresSolver.h>" namespace "OpenMS::NonNegativeLeastSquaresSolver":
    cdef enum RETURN_STATUS "OpenMS::NonNegativeLeastSquaresSolver::RETURN_STATUS":
        #wrap-attach:
        #    NonNegativeLeastSquaresSolver
        SOLVED
        ITERATION_EXCEEDED




from Types cimport *
from DataValue cimport *
from Feature cimport *
from AASequence cimport *
from PeptideEvidence cimport *
from MetaInfoInterface cimport *

cdef extern from "<OpenMS/METADATA/PeptideHit.h>" namespace "OpenMS":

    cdef cppclass PeptideHit(MetaInfoInterface):
        # wrap-inherits:
        #    MetaInfoInterface

        PeptideHit() nogil except +

        PeptideHit(double score,
                   UInt       rank,
                   Int        charge,
                   AASequence sequence
                   ) nogil except +

        PeptideHit(PeptideHit) nogil except + # wrap-ignore

        float getScore() nogil except +
        UInt getRank() nogil except +
        AASequence getSequence() nogil except +
        Int getCharge() nogil except +
        libcpp_vector[PeptideEvidence] getPeptideEvidences() nogil except +
        void setPeptideEvidences(libcpp_vector[PeptideEvidence]) nogil except +
        void addPeptideEvidence(PeptideEvidence) nogil except +
        libcpp_set[String] extractProteinAccessionsSet() nogil except +

        void setAnalysisResults(libcpp_vector[PeptideHit_AnalysisResult] aresult) nogil except +
        void addAnalysisResults(PeptideHit_AnalysisResult aresult) nogil except +
        libcpp_vector[PeptideHit_AnalysisResult] getAnalysisResults() nogil except +

        void setPeakAnnotations(libcpp_vector[PeptideHit_PeakAnnotation]) nogil except +
        libcpp_vector[PeptideHit_PeakAnnotation] getPeakAnnotations() nogil except +

        void setScore(double) nogil except +
        void setRank(UInt) nogil except +
        void setSequence(AASequence) nogil except +
        void setCharge(Int) nogil except +

        bool operator==(PeptideHit) nogil except +
        bool operator!=(PeptideHit) nogil except +

    cdef cppclass PeptideHit_AnalysisResult "OpenMS::PeptideHit::PepXMLAnalysisResult":

        PeptideHit_AnalysisResult() nogil except +
        PeptideHit_AnalysisResult(PeptideHit_AnalysisResult) nogil except + #wrap-ignore

        String score_type
        bool higher_is_better
        double main_score
        libcpp_map[String, double] sub_scores


    cdef cppclass PeptideHit_PeakAnnotation "OpenMS::PeptideHit::PeakAnnotation":

        PeptideHit_PeakAnnotation() nogil except +
        PeptideHit_PeakAnnotation(PeptideHit_PeakAnnotation) nogil except + # wrap-ignore

        String annotation
        int charge
        double mz
        double intensity

        void writePeakAnnotationsString_(String & annotation_string, libcpp_vector[ PeptideHit_PeakAnnotation ] annotations) nogil except +

        bool operator<(PeptideHit_PeakAnnotation) nogil except +
        bool operator==(PeptideHit_PeakAnnotation) nogil except +




from MSExperiment  cimport *
from MSSpectrum  cimport *
from ChromatogramPeak cimport *
from Peak1D cimport *
from String cimport *
from ProgressLogger cimport *
from streampos cimport *

cdef extern from "<OpenMS/FORMAT/CachedMzML.h>" namespace "OpenMS":

    cdef cppclass CachedmzML:

        CachedmzML() nogil except +
        CachedmzML(CachedmzML) nogil except +
        CachedmzML(String filename) nogil except +

        Size getNrSpectra() nogil except +
        Size getNrChromatograms() nogil except +

        MSSpectrum getSpectrum(Size idx) nogil except +
        MSChromatogram getChromatogram(Size idx) nogil except +

        # COMMENT: only retrieves experiment meta data (no actual data in spectra/chromatograms)
        # COMMENT: useful for filtering by attributes to then retrieve data
        MSExperiment getMetaData() nogil except +

# COMMENT: wrap static methods
cdef extern from "<OpenMS/FORMAT/CachedMzML.h>" namespace "OpenMS::CachedmzML":
    
    void store(const String& filename, MSExperiment exp) nogil except + # wrap-attach:CachedmzML
    void load(const String& filename, CachedmzML& exp) nogil except + # wrap-attach:CachedmzML




from Types cimport *
from IsobaricQuantitationMethod cimport *

cdef extern from "<OpenMS/ANALYSIS/QUANTITATION/TMTElevenPlexQuantitationMethod.h>" namespace "OpenMS":

    cdef cppclass TMTElevenPlexQuantitationMethod(IsobaricQuantitationMethod) :
        # wrap-inherits:
        #  IsobaricQuantitationMethod
        TMTElevenPlexQuantitationMethod() nogil except +
        TMTElevenPlexQuantitationMethod(TMTElevenPlexQuantitationMethod) nogil except +

